from pycolorlogs.colors import ( AscIIColors as AscIIColors,
    ColorFormatter as ColorFormatter,
    init_logs as init_logs
    )

from logging import (
    # logging level constants
    INFO      as INFO, 
    DEBUG     as DEBUG,
    ERROR     as ERROR, 
    FATAL     as FATAL, 
    CRITICAL  as CRITICAL, 
    WARNING   as WARNING, 
    WARN      as WARN,
    # logging functions
    info      as info, 
    debug     as debug, 
    error     as error, 
    warning   as warning,
    warn      as warn,
    critical  as critical, 
    fatal     as fatal,
    exception as exception)